@@ -25,7 +25,6 @@
 #ifndef GNOME_VFS_ASYNC_OPS_H
 #define GNOME_VFS_ASYNC_OPS_H
 
-#include <glib/gconvert.h> /* remove when giochannel.h is fixed */
 #include <glib/giochannel.h>
 #include <libgnomevfs/gnome-vfs-file-info.h>
 #include <libgnomevfs/gnome-vfs-find-directory.h>
@@ -34,6 +33,10 @@
 
 G_BEGIN_DECLS
 
+#define GNOME_VFS_PRIORITY_MIN     -10
+#define GNOME_VFS_PRIORITY_MAX     10
+#define GNOME_VFS_PRIORITY_DEFAULT 0
+
 typedef struct GnomeVFSAsyncHandle GnomeVFSAsyncHandle;
 
 typedef void	(* GnomeVFSAsyncCallback)	(GnomeVFSAsyncHandle *handle,
@@ -108,23 +111,27 @@ void           gnome_vfs_async_cancel                 (GnomeVFSAsyncHandle
 void           gnome_vfs_async_open                   (GnomeVFSAsyncHandle                  **handle_return,
 						       const gchar                           *text_uri,
 						       GnomeVFSOpenMode                       open_mode,
+						       int				      priority,
 						       GnomeVFSAsyncOpenCallback              callback,
 						       gpointer                               callback_data);
 void           gnome_vfs_async_open_uri               (GnomeVFSAsyncHandle                  **handle_return,
 						       GnomeVFSURI                           *uri,
 						       GnomeVFSOpenMode                       open_mode,
+						       int				      priority,
 						       GnomeVFSAsyncOpenCallback              callback,
 						       gpointer                               callback_data);
 void           gnome_vfs_async_open_as_channel        (GnomeVFSAsyncHandle                  **handle_return,
 						       const gchar                           *text_uri,
 						       GnomeVFSOpenMode                       open_mode,
 						       guint                                  advised_block_size,
+						       int				      priority,
 						       GnomeVFSAsyncOpenAsChannelCallback     callback,
 						       gpointer                               callback_data);
 void           gnome_vfs_async_open_uri_as_channel    (GnomeVFSAsyncHandle                  **handle_return,
 						       GnomeVFSURI                           *uri,
 						       GnomeVFSOpenMode                       open_mode,
 						       guint                                  advised_block_size,
+						       int				      priority,
 						       GnomeVFSAsyncOpenAsChannelCallback     callback,
 						       gpointer                               callback_data);
 void           gnome_vfs_async_create                 (GnomeVFSAsyncHandle                  **handle_return,
@@ -132,6 +139,7 @@ void           gnome_vfs_async_create                 (GnomeVFSAsyncHandle
 						       GnomeVFSOpenMode                       open_mode,
 						       gboolean                               exclusive,
 						       guint                                  perm,
+						       int				      priority,
 						       GnomeVFSAsyncOpenCallback              callback,
 						       gpointer                               callback_data);
 void           gnome_vfs_async_create_uri             (GnomeVFSAsyncHandle                  **handle_return,
@@ -139,11 +147,13 @@ void           gnome_vfs_async_create_uri             (GnomeVFSAsyncHandle
 						       GnomeVFSOpenMode                       open_mode,
 						       gboolean                               exclusive,
 						       guint                                  perm,
+						       int				      priority,
 						       GnomeVFSAsyncOpenCallback              callback,
 						       gpointer                               callback_data);
 void           gnome_vfs_async_create_symbolic_link   (GnomeVFSAsyncHandle                  **handle_return,
 						       GnomeVFSURI                           *uri,
 						       const gchar                           *uri_reference,
+						       int				      priority,
 						       GnomeVFSAsyncOpenCallback              callback,
 						       gpointer                               callback_data);
 void           gnome_vfs_async_create_as_channel      (GnomeVFSAsyncHandle                  **handle_return,
@@ -151,6 +161,7 @@ void           gnome_vfs_async_create_as_channel      (GnomeVFSAsyncHandle
 						       GnomeVFSOpenMode                       open_mode,
 						       gboolean                               exclusive,
 						       guint                                  perm,
+						       int				      priority,
 						       GnomeVFSAsyncCreateAsChannelCallback   callback,
 						       gpointer                               callback_data);
 void           gnome_vfs_async_create_uri_as_channel  (GnomeVFSAsyncHandle                  **handle_return,
@@ -158,6 +169,7 @@ void           gnome_vfs_async_create_uri_as_channel  (GnomeVFSAsyncHandle
 						       GnomeVFSOpenMode                       open_mode,
 						       gboolean                               exclusive,
 						       guint                                  perm,
+						       int				      priority,
 						       GnomeVFSAsyncCreateAsChannelCallback   callback,
 						       gpointer                               callback_data);
 void           gnome_vfs_async_close                  (GnomeVFSAsyncHandle                   *handle,
@@ -176,6 +188,7 @@ void           gnome_vfs_async_write                  (GnomeVFSAsyncHandle
 void           gnome_vfs_async_get_file_info          (GnomeVFSAsyncHandle                  **handle_return,
 						       GList                                 *uri_list,
 						       GnomeVFSFileInfoOptions                options,
+						       int				      priority,
 						       GnomeVFSAsyncGetFileInfoCallback       callback,
 						       gpointer                               callback_data);
 
@@ -191,18 +204,21 @@ void           gnome_vfs_async_set_file_info          (GnomeVFSAsyncHandle
 						       GnomeVFSFileInfo                      *info,
 						       GnomeVFSSetFileInfoMask                mask,
 						       GnomeVFSFileInfoOptions                options,
+						       int				      priority,
 						       GnomeVFSAsyncSetFileInfoCallback       callback,
 						       gpointer                               callback_data);
 void           gnome_vfs_async_load_directory         (GnomeVFSAsyncHandle                  **handle_return,
 						       const gchar                           *text_uri,
 						       GnomeVFSFileInfoOptions                options,
 						       guint                                  items_per_notification,
+						       int				      priority,
 						       GnomeVFSAsyncDirectoryLoadCallback     callback,
 						       gpointer                               callback_data);
 void           gnome_vfs_async_load_directory_uri     (GnomeVFSAsyncHandle                  **handle_return,
 						       GnomeVFSURI                           *uri,
 						       GnomeVFSFileInfoOptions                options,
 						       guint                                  items_per_notification,
+						       int				      priority,
 						       GnomeVFSAsyncDirectoryLoadCallback     callback,
 						       gpointer                               callback_data);
 GnomeVFSResult gnome_vfs_async_xfer                   (GnomeVFSAsyncHandle                  **handle_return,
@@ -211,6 +227,7 @@ GnomeVFSResult gnome_vfs_async_xfer                   (GnomeVFSAsyncHandle
 						       GnomeVFSXferOptions                    xfer_options,
 						       GnomeVFSXferErrorMode                  error_mode,
 						       GnomeVFSXferOverwriteMode              overwrite_mode,
+						       int				      priority,
 						       GnomeVFSAsyncXferProgressCallback      progress_update_callback,
 						       gpointer                               update_callback_data,
 						       GnomeVFSXferProgressCallback           progress_sync_callback,
@@ -221,6 +238,7 @@ void           gnome_vfs_async_find_directory         (GnomeVFSAsyncHandle
 						       gboolean                               create_if_needed,
 						       gboolean                               find_if_needed,
 						       guint                                  permissions,
+						       int				      priority,
 						       GnomeVFSAsyncFindDirectoryCallback     callback,
 						       gpointer                               user_data);
 
