@@ -34,8 +34,8 @@
 #include "gnome-vfs-utils.h"
 #include <bonobo-activation/bonobo-activation.h>
 #include <glib/gmessages.h>
-#include <libgnomevfs-pthread/gnome-vfs-pthread.h>
-#include <libgnomevfs-pthread/gnome-vfs-job-slave.h>
+#include <libgnomevfs/gnome-vfs-pthread.h>
+#include <libgnomevfs/gnome-vfs-job-slave.h>
 
 static gboolean vfs_already_initialized = FALSE;
 G_LOCK_DEFINE_STATIC (vfs_already_initialized);
@@ -51,6 +51,10 @@ gnome_vfs_init (void)
 	G_LOCK (vfs_already_initialized);
 
 	if (!vfs_already_initialized) {
+#ifdef ENABLE_NLS
+		bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+#endif    
+
 		if (bonobo_activation_orb_get() == NULL) {
 			bonobo_activation_init (0, bogus_argv);
 		}
