@@ -24,9 +24,12 @@
 #ifndef GNOME_VFS_IOBUF_H
 #define GNOME_VFS_IOBUF_H
 
+#include <glib/gtypes.h>
 #include <libgnomevfs/gnome-vfs-file-size.h>
 #include <libgnomevfs/gnome-vfs-result.h>
 
+G_BEGIN_DECLS
+
 typedef struct GnomeVFSIOBuf GnomeVFSIOBuf;
 
 GnomeVFSIOBuf   *gnome_vfs_iobuf_new     (gint              fd);
@@ -43,4 +46,6 @@ GnomeVFSResult   gnome_vfs_iobuf_write   (GnomeVFSIOBuf    *iobuf,
 					  GnomeVFSFileSize *bytes_written);
 GnomeVFSResult   gnome_vfs_iobuf_flush   (GnomeVFSIOBuf    *iobuf);
 
+G_END_DECLS
+
 #endif /* GNOME_VFS_IOBUF_H */
