@@ -29,6 +29,7 @@
 
 #include "gnome-vfs-mime-private.h"
 #include "gnome-vfs-mime-sniff-buffer-private.h"
+#include "gnome-vfs-mime-utils.h"
 #include "gnome-vfs-module-shared.h"
 #include "gnome-vfs-ops.h"
 #include "gnome-vfs-result.h"
@@ -123,10 +124,20 @@ add_to_key (char *mime_type, char *def)
 		s = p = g_strdup (def);
 
 		while ((ext = strtok_r (s, " \t\n\r,", &tokp)) != NULL) {
-			list = (GList *) g_hash_table_lookup (mime_extensions [priority], ext);
+			gboolean found;
+			gpointer orig_key;
+
+			found = g_hash_table_lookup_extended (mime_extensions [priority], ext,
+							      &orig_key, (gpointer *)&list);
+			if (!found) {
+				orig_key = NULL;
+				list = NULL;
+			}
+			
 			if (!g_list_find_custom (list, mime_type, list_find_type)) {
 				list = g_list_prepend (list, g_strdup (mime_type));
-				g_hash_table_insert (mime_extensions [priority], g_strdup (ext), list);
+				g_hash_table_insert (mime_extensions [priority],
+						     found ? orig_key : g_strdup (ext), list);
 			}
 			s = NULL;
 		}
@@ -353,7 +364,7 @@ mime_init (void)
 
 	mime_data_date_tracker = gnome_vfs_file_date_tracker_new ();
 	
-	gnome_mime_dir.dirname = g_strconcat (GNOME_VFS_DATADIR, "/mime-info", NULL);
+	gnome_mime_dir.dirname = g_strdup (DATADIR "/mime-info");
 	gnome_mime_dir.system_dir = TRUE;
 
 	user_mime_dir.dirname = g_strconcat (g_get_home_dir (), "/.gnome/mime-info", NULL);
@@ -515,15 +526,27 @@ gnome_vfs_get_mime_type_internal (GnomeVFSMimeSniffBuffer *buffer, const char *f
 			return result;
 		}
 		
-		/* Special handling of gzip files -- use the file name to make a more
-		 * accurate guess of the file type for formats such as gnumeric.gz and
-		 * pdf.gz. Without this, these would always get identified as gzip even though
-		 * their name would suggest otherwise.
+		/* So many file types come compressed by gzip that extensions are
+		 * more reliable than magic typing. If the file has a suffix, then
+		 * use the type from the suffix.
+		 *
 		 * FIXME bugzilla.eazel.com 6867:
-		 * Generalize this so that we can have different magic patters
-		 * other than gzip do this.
+		 * Allow specific mime types to override magic detection
 		 */
 		if (gnome_vfs_sniff_buffer_looks_like_gzip (buffer, file_name)) {
+			/* gzip -- treat extensions as a more accurate source
+			 * of type information.
+			 */
+			
+			if (file_name != NULL) {
+				result = gnome_vfs_mime_type_from_name_or_default (file_name, NULL);
+			}
+			
+			if (result != NULL) {
+				return result;
+			}
+			
+			/* Didn't find an extension match, assume gzip. */
 			return "application/x-gzip";
 		}
 		
@@ -563,7 +586,7 @@ gnome_vfs_get_mime_type_internal (GnomeVFSMimeSniffBuffer *buffer, const char *f
 }
 
 /**
- * gnome_vfs_get_mime_type:
+ * gnome_vfs_get_mime_type_common:
  * @uri: a real file or a non-existent uri.
  *
  * Tries to guess the mime type of the file represented by @uir.
@@ -578,7 +601,7 @@ gnome_vfs_get_mime_type_internal (GnomeVFSMimeSniffBuffer *buffer, const char *f
  * 
  */
 const char *
-gnome_vfs_get_mime_type (GnomeVFSURI *uri)
+gnome_vfs_get_mime_type_common (GnomeVFSURI *uri)
 {
 	const char *result;
 	char *base_name;
@@ -961,3 +984,38 @@ gnome_vfs_file_date_tracker_date_has_changed (FileDateTracker *tracker)
 
 	return any_date_changed;
 }
+
+
+
+
+/**
+ * gnome_vfs_get_mime_type:
+ * @text_uri: URI of the file for which to get the mime type
+ * 
+ * Determine the mime type of @text_uri. The mime type is determined
+ * in the same way as by gnome_vfs_get_file_info(). This is meant as
+ * a convenience function for times when you only want the mime type.
+ * 
+ * Return value: The mime type, or NULL if there is an error reading 
+ * the file.
+ **/
+char *
+gnome_vfs_get_mime_type (const char *text_uri)
+{
+	GnomeVFSFileInfo *info;
+	char *mime_type;
+	GnomeVFSResult result;
+
+	info = gnome_vfs_file_info_new ();
+	result = gnome_vfs_get_file_info (text_uri, info,
+					  GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
+					  GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
+	if (info->mime_type == NULL || result != GNOME_VFS_OK) {
+		mime_type = NULL;
+	} else {
+		mime_type = g_strdup (info->mime_type);
+	}
+	gnome_vfs_file_info_unref (info);
+
+	return mime_type;
+}
